// -- Ray Pong
// games.neoclaw.net/raypong

//main game loop
//Luke Lenhart, 2008
//See the MPMA Framework license for details on how this code may be used.

#include <iostream>

#include "mpma/base/Setup.h"
#include "mpma/base/Memory.h"
#include "mpma/base/Thread.h"
#include "mpma/base/DebugRouter.h"
#include "mpma/base/Profiler.h"
#include "mpma/gfxsetup/Setup.h"

#include "Game.h"
#include "Render.h"

void MemoryErrorReport(const std::string &msg)
{
    std::cout<<msg;
    std::cout.flush();
}

int main()
{
    //init framework
    std::cout<<"Setting up..."<<std::endl;
    MPMA::InitAndShutdown frameworkInit;

    mpmaMemoryManager.AddLeakReportCallback(MemoryErrorReport);
    MPMA::RouterOutputStdout *stdReportOutput=new3(MPMA::RouterOutputStdout());
    MPMA::ErrorReport().AddOutputMethod(stdReportOutput);

    //create the window
    GFX::GraphicsSetup gfx;
    gfx.Name="Ray Pong";
#ifdef _DEBUG
    gfx.Width=100;
    gfx.Height=100;
#else
    gfx.Width=400;
    gfx.Height=400;
#endif
    SetupWindow(gfx);

    //main loop
    std::cout<<"Running..."<<std::endl;
    for ( ;GFX::GetWindowState() ;)
    {
        ProfileScope("Frame Update");

        Game::Update();
        Renderer::DrawScene();
        GFX::UpdateWindow();

        MPMA::Sleep(0); //prevent stalling the entire system
    }

    //clean up
    std::cout<<"Shutting down..."<<std::endl;
    Renderer::Cleanup();
    GFX::ShutdownWindow();
    delete3(stdReportOutput);

    return 0;
}

