//File name converter and file management functions
//Luke Lenhart (2007)
//See /docs/License.txt for details on how this code may be used.

#include "File.h"
#include <algorithm>

namespace MPMA
{
    //remove rediculousness from a filename
    void Filename::PurifyFilename()
    {
        //strip out adjacent / or \ characters
        std::string newFname;
        newFname.reserve(fname.size());
        bool prevSlash=false;
        for (std::string::iterator i=fname.begin(); i!=fname.end(); ++i)
        {
            bool curSlash=*i=='/' || *i=='\\';

            if (curSlash)
            {
                if (!prevSlash)
                    newFname.push_back(*i);
            }
            else
            {
                newFname.push_back(*i);
            }

            prevSlash=curSlash;
        }

        fname=newFname;
    }

    //Gets the path containing this file.
    Directory Filename::GetPath() const
    {
        std::string path=fname;
        while (!path.empty() && path[path.size()-1]!='\\' && path[path.size()-1]!='/')
            path.resize(path.size()-1);
        return Directory(path);
    }

    //Gets the name of this file, without the path.
    std::string Filename::GetFilenameNoPath(bool includeExtension) const
    {
        int len=fname.size()-((const std::string&)GetPath()).size();
        int start=fname.size()-len;
        std::string cur=fname.substr(start, len);

        if (!includeExtension)
        {
            std::string ext=GetFilenameExtension();
            if (ext.size()>=1)
                cur.resize(cur.size()-ext.size()-1);
        }

        return cur;
    }

    //Gets the extension of a filename
    std::string Filename::GetFilenameExtension() const
    {
        std::string ext;
        if (fname.empty())
            return ext;

        std::string revext;
        for (int i=fname.size()-1; i>=0; --i)
        {
            if (fname[i]=='.' || fname[i]=='/' || fname[i]=='\\')
                break;

            revext.push_back(fname[i]);
        }

        for (std::string::reverse_iterator i=revext.rbegin(); i!=revext.rend(); ++i)
            ext.push_back(*i);

        return ext;
    }

    // --

    //ctor
    Directory::Directory()
    {
    }
    Directory::Directory(const Filename &initialPath)
    {
        path=initialPath;
    }

    //change to a local-relative directory
    void Directory::StepOut()
    {

        if (path.empty())  //if no path is set (so "current")
        {
            path="..";
        }
        else if (path.size()>=2 && path[path.size()-1]=='.' && path[path.size()-2]=='.') //if the last path chunk is ".."
        {
            path+="/..";
        }
        else //normal case, strip off the last path chunk
        {
            while (!path.empty())
            {
                char c=path[path.size()-1];
                path.resize(path.size()-1);
                if (c=='/' || c=='\\')
                    break;
            }
        }
    }
    void Directory::StepInto(const std::string &subdir)
    {
        if (!path.empty())
            path+="/"+subdir;
        else
            path=subdir;
    }
}
