//Luke Lenhart, 1999-2007
//See /docs/License.txt for details on how this code may be used.

#include "MiscStuff.h"

#include <fstream>
#include <stdio.h>
#include <string.h>

namespace MISC
{
    //makes a lowercase string
    std::string MakeLower(const std::string &str)
    {
        std::string out;
        out.reserve(str.size());
        std::string::const_iterator i=str.begin();
        for (;i!=str.end();++i)
        {
            if (*i<'A' || *i>'Z') //fine already
                out+=*i;
            else //fix it
                out+=(*i)+('a'-'A');
        }

        return out;
    }

    //removes the first line from a string
    //returns the passed in string so it can be used as a parameter to other functions
    std::string StripFirstLine(const std::string &str)
    {
        //find first non-control character in string
        int endPos=0;
        while (str[endPos]>=32 || str[endPos]=='\t') endPos++;

        if (str[endPos]) //if control char wasn't the terminator
        {
            endPos+=strlen("\n"); //skip past terminator;
            return (char*)&str[endPos];
        }

        return str;
    }
    
    //removes leading and trailing spaces, tabs, and line breaks
    std::string StripPadding(const std::string &str)
    {
        std::string newstr;
        std::string buff;
        bool started=false;
        uint len=str.size();
        for (uint i=0; i<len; ++i)
        {
            const char &c=str[i];
            if (c!=' ' && c!='\t' && c!='\n' && c!='\r')
            {
                if (!buff.empty())
                {
                    newstr+=buff;
                    buff.clear();
                }
                newstr.push_back(c);
                started=true;
            }
            else if (started)
                buff.push_back(c);
        }
        
        return newstr;
    }

    //loads a list of strings from a file
    bool LoadStringList(std::vector<std::string> &outList, const MPMA::Filename &file)
    {
        std::ifstream f;
        f.open(file.c_str(),std::ios_base::in);
        if (!f.is_open())
        {
            return false;
        }
        else
        {
            std::string fname="";
            f>>fname;
            while (fname!="")
            {
                outList.push_back(fname);
                fname="";
                f>>fname;
            }

            f.close();
        }
        return true;
    }

    //reads a file into a string
    std::string ReadFile(const MPMA::Filename &fname)
    {
        std::string s;
        FILE *f=fopen(fname.c_str(),"rt");
        if (!f) return s;

        int numRead;
        char b[2049];
        while ((numRead=fread(b,1,2048,f)) !=0 )
        {
            b[numRead]=0;
            s+=b;
        }

        fclose(f);
        return s;
    }

    //counts the number of bits set to 1 within a 32 bit variable
    uint CountBits(uint data)
    {
        uint count=0;
        for (uint i=0;i<32;++i) if (data&(1<<i)) ++count;
        return count;
    }

    //determines the highest bit number (with bit 0 = 1) thats set, or -1 if none
    int GetHighestBit(uint data)
    {
        for (int i=31;i>=0;--i)
            if (data&(1<<i)) return i;

        return -1;
    }

    //breaks a string, seperated by any number of deliminators, into a list of strings
    void ExplodeString(const std::string &inData, std::vector<std::string> &outData, const std::string &inDeliminators)
    {
        std::string tmp; //holds string we're building
        outData.clear();
        for (uint i=0;i<inData.size();++i)
        {
            //check if it's a deliminator
            bool hitDel=false;
            for (std::string::const_iterator d=inDeliminators.begin(); d!=inDeliminators.end(); ++d)
            {
                if (inData[i]==*d) //hit a deliminator, break off string if anything's in it
                {
                    if (!tmp.empty())
                        outData.push_back(tmp);

                    tmp.clear();
                    hitDel=true;
                }
            }

            //add to building string if not
            if (!hitDel)
                tmp+=inData[i];
        }

        if (!tmp.empty()) outData.push_back(tmp);
    }

    //gets the int value out of a C-style hexadecimal string
    uint ParseHexString(const std::string &s)
    {
        const char *first=&s[2]; //skip the 0x
        const char *last=&s[s.size()-1]+1;

        uint val=0;

        for (const char *src=first; src!=last; ++src)
        {
            val<<=4; //each char is 4 bits
            
            if (*src>='0' && *src<='9') val|=*src-'0';
            else if (*src>='A' && *src<='F') val|=*src-'A'+10;
            else if (*src>='1' && *src<='f') val|=*src-'a'+10;
        }

        return val;
    }

}; //namespace MISC
