//!\file Unified.h Unified input for keyboard+mouse+gamepad.  Note that all input relies on the window being set up first.
//Luke Lenhart, 2008
//See /docs/License.txt for details on how this code may be used.

#pragma once
#include "../base/Types.h"
#include <string>
#include <vector>

//!General Input.
namespace INPUT
{
    //Note that all input relies on the window being set up first.

    //!The device that a button represents
    enum DeviceType
    {
        KEYBOARD_DEVICE=1,
        MOUSE_DEVICE,
        GAMEPAD_DEVICE
    };

    // -- buttons

    //!Represents an input button from an arbitrary device
    struct UnifiedButton
    {
        //!The source device of the button
        DeviceType Type;
        //!Data specific to the device for this button.  The application should not try to interpret this value.
        uint32 Data;

        //!ctor
        inline UnifiedButton(DeviceType type, uint32 data)
        {
            Type=type;
            Data=data;
        }

        inline bool operator==(const UnifiedButton &o) const //!<==
                { return Type==o.Type && Data==o.Data; }
        inline bool operator!=(const UnifiedButton &o) const //!<!=
                { return !(*this==o); }
    };

    //!Returns whether the specific button is currently pressed down
    bool IsButtonDown(const UnifiedButton &button);

    //!Returns a list of buttons that are currently pressed down
    const std::vector<UnifiedButton>& GetCurrentlyPressedButtons();

    //!Returns a list of buttons that were pressed since the last frame
    const std::vector<UnifiedButton>& GetNewlyPressedButtons();

    //!Retrieves a user-friendly name for a unified button
    const std::string& GetFriendlyName(const UnifiedButton &button);


    // -- directions
    //(work in progress below)
/*
    //!Uniquely identifies an axis contraption.  Any given device type may have more than one set of axes
    struct UnifiedAxisIdentifier
    {
        //!The source device of the axis.
        DeviceType Type;

        //!An identifier for the axis.  The application should not try to interpret this value.
        uint32 Identifier;
    };

    //!Represents the state of a specific set of axes.  Most devices only make use of X and Y
    struct UnifiedAxisState
    {
        //!A value from -1(left) to 1(right) for the current state of the x axis
        float X;

        //!A value from -1(down) to 1(up) for the current state of the y axis
        float Y;

        //!A value from -1(below) to 1(above) for the current state of the z axis
        float Z;
    };

    //!Returns a list of all axis sets that can be polled
    const std::vector<UnifiedAxisIdentifier>& GetAxisList();

    //!Returns the current state of a specific axis set
    UnifiedAxisState GetAxisState(const UnifiedAxisIdentifier &axis);

    //!Returns the sum of all axes from all devices as a single value
    UnifiedAxisState GetCombinedAxisState();

    //!Retrieves a user-friendly name for unified axis set
    const std::string& GetFriendlyName(const UnifiedAxisIdentifier &axis);
*/
}
